@@ -7,6 +7,7 @@ Semantic MediaWiki wrapper
 
 import string
 import base64
+import urllib
 import urllib2
 import traceback
 import socket
@@ -16,9 +17,9 @@ import rdflib
 import mwclient
 from bs4 import BeautifulSoup, Comment
 from enum import Enum
+from os.path import expanduser, join
 
-
-class WikiHTTPPool(mwclient.http.HTTPPool):
+class HTTPPoolWithAuth(mwclient.http.HTTPPool):
     header_auth = {}
 
     def __init__(self, http_login, http_pass):
@@ -27,7 +28,7 @@ class WikiHTTPPool(mwclient.http.HTTPPool):
             auth = 'Basic ' + string.strip(base64.encodestring(token))
             self.header_auth['Authorization'] = auth
 
-        super(WikiHTTPPool, self).__init__()
+        super(HTTPPoolWithAuth, self).__init__()
 
     def updateHeader(self,  headers=None):
         if headers:
@@ -37,19 +38,19 @@ class WikiHTTPPool(mwclient.http.HTTPPool):
         return headers
 
     def get(self, host, path, headers=None):
-        return super(WikiHTTPPool, self).get(host, path,
+        return super(HTTPPoolWithAuth, self).get(host, path,
                                              self.updateHeader(headers))
 
     def post(self, host, path, headers=None, data=None):
-        return super(WikiHTTPPool, self).post(host, path,
+        return super(HTTPPoolWithAuth, self).post(host, path,
                                               self.updateHeader(headers), data)
 
     def head(self, host, path, headers=None, auto_redirect=False):
-        return super(WikiHTTPPool, self).head(host, path,
+        return super(HTTPPoolWithAuth, self).head(host, path,
                                               self.updateHeader(headers), auto_redirect)
 
     def request(self, method, host, path, headers, data, raise_on_not_ok, auto_redirect):
-        return super(WikiHTTPPool, self).request(method, host, path,
+        return super(HTTPPoolWithAuth, self).request(method, host, path,
                                                  self.updateHeader(headers), data, raise_on_not_ok, auto_redirect)
 
 
@@ -71,14 +72,58 @@ class SemanticMediaWiki(object):
     rdf_store = None
     rdf_session = None
     wiki_status = smw_error.NONE
+    config = None
+
+    """
+    default config file is in ~/.smwrc
+
+    config = {
+        "host": "www.foo.com",
+        "path": "/wiki/",
+        "http_login": null,
+        "http_pass": null,
+        "wiki_login": null,
+        "wiki_pass": null
+    }
+    """
+
+    @staticmethod
+    def from_config(config=None):
+        if not config:
+            config_file = join(expanduser("~"), '.smwrc')
+            try:
+                with open(config_file, 'r') as f:
+                    config = json.load(f)
+            except IOError:
+                print config_file, "does not exist"
+                return None
+            except ValueError:
+                print config_file, "is not a valid json"
+                return None
+
+        wiki = SemanticMediaWiki(
+            host = config["host"],
+            path = config.get("path", '/'),
+            http_login = config.get("http_login", None),
+            http_pass = config.get("http_pass", None),
+            wiki_login = config.get("wiki_login", None),
+            wiki_pass = config.get("wiki_pass", None))
+        return wiki
 
     def __init__(self, host, path="/",
                  http_login=None, http_pass=None,
                  wiki_login=None, wiki_pass=None):
-
+        self.config = {
+            "host": host,
+            "path": path,
+            "http_login": http_login,
+            "http_pass": http_pass,
+            "wiki_login": wiki_login,
+            "wiki_pass": wiki_login
+        }
         try:
             self.site = mwclient.Site(host, path,
-                                      pool=WikiHTTPPool(http_login, http_pass))
+                                      pool=HTTPPoolWithAuth(http_login, http_pass))
             self.conn = self.site.connection.find_connection(host)
             # print self.site.connection
             # print self.site.connection.__class__.__name__
@@ -238,13 +283,13 @@ class SemanticMediaWiki(object):
                 property = property.replace("wiki:Property:", "", 1)
             if isinstance(o,  rdflib.URIRef):
                 if property in ["swivt:page", "rdfs:isDefinedBy"]:
-                    object = o
+                    object = str(o)
                 else:
                     object = self.unescapeSMW(nsg.normalizeUri(o))
                     if object.startswith("wiki:"):
                         object = object.replace("wiki:", "", 1)
             else:
-                object = o
+                object = str(o)
 
             if property.startswith("swivt:specialProperty"):
                 continue
@@ -258,3 +303,25 @@ class SemanticMediaWiki(object):
         # @TODO: handle internal objects
 
         return json_object
+
+    def list(self, prefix=None):
+        """
+        list page with given prefix. up to 5000 pages may be returned
+
+        site.allpages() fails on wiki runs on HHVM
+        #allpages = list(wiki.site.allpages(prefix='Foo-'))
+        """
+        para = {
+            "action": 'query',
+            'list': 'allpages',
+            'format': 'json',
+            'aplimit': '5000'
+        }
+        if prefix:
+            para['apprefix'] = prefix
+        url =  self.site.path +  r"api.php?" + urllib.urlencode(para)
+        r = self.get(url)
+        r = json.loads(r)
+        r = r['query']['allpages']
+        allpages = [p['title'] for p in r]
+        return allpages
\ No newline at end of file
